package projet.iut.jeu_de_la_vie.view;

import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Rect;
import android.os.Build;
import android.util.AttributeSet;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.annotation.RequiresApi;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

import projet.iut.jeu_de_la_vie.model.cellule.Cellule;
import projet.iut.jeu_de_la_vie.model.cellule.Position;
import projet.iut.jeu_de_la_vie.model.cellule.observer.ObserverCellule;
import projet.iut.jeu_de_la_vie.model.cellulesVivantes.CellulesVivantes;
import projet.iut.jeu_de_la_vie.model.cellulesVivantes.observer.ObservableCV;
import projet.iut.jeu_de_la_vie.model.cellulesVivantes.observer.ObserverCV;

public class PlateauView extends View implements ObserverCV {
    private HashMap<Position, Integer> colors;

    public PlateauView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs);
    }

    public PlateauView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
    }

    @RequiresApi(api = Build.VERSION_CODES.LOLLIPOP)
    public PlateauView(Context context, @Nullable AttributeSet attrs, int defStyleAttr, int defStyleRes) {
        super(context, attrs, defStyleAttr, defStyleRes);
    }

    private Paint paint;
    private int colones;
    private int lignes;
    private int dethColor;

    public PlateauView(Context context, int colones, int lignes){
        super(context);
        colors = new HashMap<>();
        paint = new Paint();
        this.colones = colones;
        this.lignes = lignes;
        this.dethColor = Color.BLACK;
    }

    @Override
    public void onDraw(Canvas canvas){
        super.onDraw(canvas);
        Integer tmp;
        Rect rect;
        for(int i=0; i<lignes; ++i){
            for(int j=0; j<colones; ++j){
                Position p = new Position(j, i);
                tmp = colors.get(p);
                paint.setColor(dethColor);
                if(tmp instanceof Integer){
                    paint.setColor(tmp);
                }
                rect = new Rect(j, i, j+10, i+10);
                canvas.drawRect(rect, paint);
            }
        }
    }


    @Override
    public void update(Cellule changingCell) {
        if(changingCell.isAlive()){
            colors.put(changingCell.getPosition(), changingCell.getActiveColor());
        }
        else {
            colors.remove(changingCell.getPosition());
        }
    }

    public void setColones(int value) throws IllegalArgumentException{
        if(value < 0){
            throw new IllegalArgumentException("La valeur des colones doit être supperieur à 0");
        }
        colones = value;
    }

    public void setLignes(int value) throws IllegalArgumentException{
        if(value < 0){
            throw new IllegalArgumentException("La valeur des colones doit être supperieur à 0");
        }
        lignes = value;
    }
}
